/* -*- mode: c++; c-basic-offset:4 -*-
    models/keylistmodelinterface.cpp

    This file is part of libkleo, the KDE keymanagement library
    SPDX-FileCopyrightText: 2021 g10 Code GmbH

    SPDX-License-Identifier: GPL-2.0-or-later
*/

#include <config-libkleo.h>

#include "keylistmodelinterface.h"

Kleo::KeyListModelInterface::~KeyListModelInterface()
{
}
